﻿using System.Windows;
using WickedFlame.VecDraw.Base.mvvm.common;
using WickedFlame.VecDraw.Base.mvvm.mediator;

namespace WickedFlame.VecDraw.Base.mvvm
{
    /// <summary>
    /// Base class for all view models
    /// </summary>
    public class BaseViewModel : NotifyPropertyChangedBase
    {
        public BaseViewModel(DependencyObject parent)
        {
            if (Designer.IsDesignMode)
                return;

            Parent = parent;

			//Register all decorated methods to the Mediator
			Mediator.Register(this);
        }

        public DependencyObject Parent
        {
            get;
            private set;
        }

        public virtual void Loaded()
        {
        }

		public virtual void Unload()
		{
		}

    	#region ServiceLocator

        ServiceLocator serviceLocator = new ServiceLocator();

        /// <summary>
        /// Gets the service locator 
        /// </summary>
        public ServiceLocator ServiceLocator
        {
            get 
            {
                return serviceLocator; 
            }
        }

        /// <summary>
        /// Gets a service from the service locator
        /// </summary>
        /// <typeparam name="T">The type of service to return</typeparam>
        /// <returns>Returns a service that was registered with the Type T</returns>
        public T GetService<T>()
        {
            return serviceLocator.GetService<T>();
        }

        #endregion

		#region Mediator

		static readonly Mediator mediator = new Mediator();

		public Mediator Mediator
		{
			get { return mediator; }
		}

		#endregion

	}
}
