﻿using System;

namespace WickedFlame.Core.History
{
    /// <summary>
    /// Abstract base class for commands used for Undo - Redo 
    /// Subclasses contain enough information to make Undo and Redo operations for every specific command.
    /// Based on the Command Patern.
    /// </summary>
    public abstract class HistoryCommand
    {
        int _layerIndex;
        /// <summary>
        /// Gets the index of the layer that has a state changed
        /// </summary>
        public int LayerIndex
        {
            get
            {
                return _layerIndex;
            }
        }

        string _title = "";
        public string Title
        {
            get
            {
                return _title;
            }
            protected set
            {
                _title = value;
            }
        }

        public HistoryCommand(int layerIndex)
        {
            _layerIndex = layerIndex;
        }

        /// <summary>
        /// This function is used to make Undo operation.
        /// It makes action opposite to the original command.
        /// </summary>
        /// <param name="list"></param>
        public abstract void Undo(ShapeList list);

        /// <summary>
        /// This command is used to make Redo operation.
        /// It makes original command again.
        /// </summary>
        /// <param name="list"></param>
        public abstract void Redo(ShapeList list);
    }
}
