﻿using System;
using System.Collections.Generic;
using WickedFlame.Core.Shapes;
using WickedFlame.Core.State;

namespace WickedFlame.Core.History
{
    public class PropertyChangedCommand: HistoryCommand
    {
        readonly List<VectorShape> _selectedShapes;
        readonly Dictionary<VectorShape, ShapeMemento> _originalStates;
        Dictionary<VectorShape, ShapeMemento> _newStates;

        public PropertyChangedCommand(ShapeList shapelist, int layerIndex)
            : base(layerIndex)
        {
            Title = "Shape Properties changed";

            _selectedShapes = new List<VectorShape>();
            foreach (VectorShape shape in shapelist.Selection)
            {
                _selectedShapes.Add(shape);
            }

            _originalStates = new Dictionary<VectorShape,ShapeMemento>();

            foreach (VectorShape shape in _selectedShapes)
            {
                if (shape is IStateCapturable)
                {
                    _originalStates.Add(shape, ((IStateCapturable)shape).CreateMementoState());
                }
            }
        }

        /// <summary>
        /// sets the new state of the shapes in the selection list
        /// </summary>
        public void SetNewState()
        {
            _newStates = new Dictionary<VectorShape, ShapeMemento>();
            
            foreach (VectorShape shape in _selectedShapes)
            {
                if (shape is IStateCapturable)
                {
                    _newStates.Add(shape, ((IStateCapturable)shape).CreateMementoState());
                }
            }
        }

        public override void Undo(ShapeList list)
        {
            foreach (VectorShape shape in _selectedShapes)
            {
                if (_originalStates.ContainsKey(shape))
                {
                    if (shape is IStateCapturable)
                    {
                        ((IStateCapturable)shape).ResetState(_originalStates[shape]);
                    }
                }
            }
        }

        public override void Redo(ShapeList list)
        {
            foreach (VectorShape shape in _selectedShapes)
            {
                if (_newStates.ContainsKey(shape))
                {
                    if (shape is IStateCapturable)
                    {
                        ((IStateCapturable)shape).ResetState(_newStates[shape]);
                    }
                }
            }
        }
    }
}
