﻿using System;
using System.Windows.Input;

namespace WickedFlame.Core.Tools
{
	/// <summary>
	/// Base class for all tools
	/// </summary>
	public abstract class VectorTool
	{
        public VectorTool()
        {
        }

		/// <summary>
		/// Left mouse button is pressed
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		public virtual void OnMouseLeftButtonDown(DrawingCanvas sender, MouseButtonEventArgs e)
		{
		}

		/// <summary>
		/// Left mouse button is released
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
        public virtual void OnMouseLeftButtonUp(DrawingCanvas sender, MouseButtonEventArgs e)
		{
		}

        /// <summary>
        /// Right mouse button is released
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        public virtual void OnMouseRightButtonUp(DrawingCanvas sender, MouseButtonEventArgs e)
        {
        }

		/// <summary>
		/// Mouse is moved, left mouse button is pressed or none button is pressed
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
        public virtual void OnMouseMove(DrawingCanvas sender, MouseEventArgs e)
		{
		}

        public virtual void OnKeyDown(DrawingCanvas sender, KeyEventArgs e)
        {
        }

        public virtual void OnTextInput(DrawingCanvas sender, TextCompositionEventArgs e)
        {
        }

        /// <summary>
        /// override this method if the tool needs activation
        /// </summary>
        /// <param name="canvas"></param>
        public virtual void OnActivate(DrawingCanvas canvas)
        {
        }

        /// <summary>
        /// override this method if the tool needs cleaning up
        /// </summary>
        public virtual void OnDeactivate(DrawingCanvas sender)
        {
        }
	}
}
